/**
 * Copyright (c) 2008 Washington University
 */
package org.nrg.xnat.filestore;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Map;
import java.util.LinkedHashMap;

import org.apache.commons.lang.StringEscapeUtils;
import org.nrg.xnat.tags.TagLabelException;


/**
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 *
 */
public class MetadataReplaceOp implements MetadataOp {
    private static final String SQL_REPLACE_METADATA = "UPDATE resource_tags SET value = ? WHERE value = ? AND tag_id = ? AND resource_id = ?";

    private final Connection c;
    private final int tagID;
    private final Map<String,String> changes;


    public MetadataReplaceOp(final Connection c, final int tagID, final Map<String,String> changes) {
	this.c = c;
	this.tagID = tagID;
	this.changes = new LinkedHashMap<String,String>(changes);
    }


    public MetadataReplaceOp(final Connection c, final String label, final Map<String,String> changes) throws SQLException,TagLabelException {
	this(c, MetadataOp.Util.getLabelID(c, label), changes);
    }


    /* (non-Javadoc)
     * @see org.nrg.xnat.filestore.MetadataOp#apply(java.net.URI)
     */
    public void apply(final int resourceID) throws SQLException {
	final PreparedStatement replace = c.prepareStatement(SQL_REPLACE_METADATA);
	try {
	    for (final Map.Entry<String,String> me : changes.entrySet()) {
		replace.setString(1, StringEscapeUtils.escapeSql(me.getValue()));	// new value
		replace.setString(2, StringEscapeUtils.escapeSql(me.getKey()));	// old value
		replace.setInt(3, tagID);
		replace.setInt(4, resourceID);
		replace.executeUpdate();	// TODO: check return value
	    }
	} finally {
	    try { replace.close(); } catch (SQLException ignore) {}
	}
    }
}
